@@ -4,7 +4,7 @@ import functools
 import operator
 import typing
 import warnings
-from typing import Any, Callable, Iterator, Mapping, Sequence, Tuple, TypeVar, Union
+from typing import Any, Callable, Generator, Iterator, Mapping, Sequence, Tuple, Union
 
 import boost_histogram as bh
 import histoprint
@@ -12,12 +12,13 @@ import numpy as np
 
 import hist
 
+from . import interop
+from ._compat.typing import ArrayLike, Protocol, Self, SupportsIndex
 from .axestuple import NamedAxesTuple
 from .axis import AxisProtocol
 from .quick_construct import MetaConstructor
 from .storage import Storage
 from .svgplots import html_hist, svg_hist_1d, svg_hist_1d_c, svg_hist_2d
-from .typing import ArrayLike, Protocol, SupportsIndex
 
 if typing.TYPE_CHECKING:
     from builtins import ellipsis
@@ -38,6 +39,7 @@ InnerIndexing = Union[
 ]
 IndexingWithMapping = Union[InnerIndexing, Mapping[Union[int, str], InnerIndexing]]
 IndexingExpr = Union[IndexingWithMapping, Tuple[IndexingWithMapping, ...]]
+AxisTypes = Union[AxisProtocol, Tuple[int, float, float]]
 
 
 # Workaround for bug in mplhep
@@ -52,7 +54,14 @@ def _proc_kw_for_lw(kwargs: Mapping[str, Any]) -> dict[str, Any]:
     }
 
 
-T = TypeVar("T", bound="BaseHist")
+def process_mistaken_quick_construct(
+    axes: Sequence[AxisTypes | hist.quick_construct.ConstructProxy],
+) -> Generator[AxisTypes, None, None]:
+    for ax in axes:
+        if isinstance(ax, hist.quick_construct.ConstructProxy):
+            yield from ax.axes
+        else:
+            yield ax
 
 
 class BaseHist(bh.Histogram, metaclass=MetaConstructor, family=hist):
@@ -60,7 +69,7 @@ class BaseHist(bh.Histogram, metaclass=MetaConstructor, family=hist):
 
     def __init__(
         self,
-        *args: AxisProtocol | Storage | str | tuple[int, float, float],
+        *in_args: AxisTypes | Storage | str,
         storage: Storage | str | None = None,
         metadata: Any = None,
         data: np.typing.NDArray[Any] | None = None,
@@ -75,17 +84,16 @@ class BaseHist(bh.Histogram, metaclass=MetaConstructor, family=hist):
         self.name = name
         self.label = label
 
-        if args and storage is None and isinstance(args[-1], (Storage, str)):
-            storage = args[-1]
-            args = args[:-1]
+        args: tuple[AxisTypes, ...]
+
+        if in_args and storage is None and isinstance(in_args[-1], (Storage, str)):
+            storage = in_args[-1]
+            args = in_args[:-1]  # type: ignore[assignment]
+        else:
+            args = in_args  # type: ignore[assignment]
 
         # Support raw Quick Construct being accidentally passed in
-        args = [  # type: ignore[assignment]
-            a.axes[0]  # type: ignore[union-attr, union-attr, union-attr, union-attr]
-            if isinstance(a, hist.quick_construct.ConstructProxy) and len(a.axes) == 1
-            else a
-            for a in args
-        ]
+        args = tuple(ax for ax in process_mistaken_quick_construct(args))
 
         if isinstance(storage, str):
             storage_str = storage.title()
@@ -95,8 +103,8 @@ class BaseHist(bh.Histogram, metaclass=MetaConstructor, family=hist):
                 storage_str = "WeightedMean"
             storage = getattr(bh.storage, storage_str)()
         elif isinstance(storage, type):
-            msg = f"Please use '{storage.__name__}()' instead of '{storage.__name__}'"
-            warnings.warn(msg)
+            msg = f"Please use '{storage.__name__}()' instead of '{storage.__name__}'"  # type: ignore[unreachable]
+            warnings.warn(msg, stacklevel=2)
             storage = storage()
 
         super().__init__(*args, storage=storage, metadata=metadata)  # type: ignore[call-overload]
@@ -107,7 +115,7 @@ class BaseHist(bh.Histogram, metaclass=MetaConstructor, family=hist):
                 disallowed_warning = (
                     f"{ax.name} is a protected keyword and cannot be used as axis name"
                 )
-                warnings.warn(disallowed_warning)
+                warnings.warn(disallowed_warning, stacklevel=2)
 
         valid_names = [ax.name for ax in self.axes if ax.name]
         if len(valid_names) != len(set(valid_names)):
@@ -141,18 +149,16 @@ class BaseHist(bh.Histogram, metaclass=MetaConstructor, family=hist):
         if self.size == 0:
             return None
 
-        if self.ndim == 1:
-            if len(self.axes[0]) <= 1000:
-                return str(
-                    html_hist(
-                        self,
-                        svg_hist_1d_c if self.axes[0].traits.circular else svg_hist_1d,
-                    )
+        if self.ndim == 1 and len(self.axes[0]) <= 1000:
+            return str(
+                html_hist(
+                    self,
+                    svg_hist_1d_c if self.axes[0].traits.circular else svg_hist_1d,
                 )
+            )
 
-        if self.ndim == 2:
-            if len(self.axes[0]) <= 200 and len(self.axes[1]) <= 200:
-                return str(html_hist(self, svg_hist_2d))
+        if self.ndim == 2 and len(self.axes[0]) <= 200 and len(self.axes[1]) <= 200:
+            return str(html_hist(self, svg_hist_2d))
 
         return None
 
@@ -169,13 +175,13 @@ class BaseHist(bh.Histogram, metaclass=MetaConstructor, family=hist):
 
     @classmethod
     def from_columns(
-        cls: type[T],
+        cls,
         data: Mapping[str, ArrayLike],
         axes: Sequence[str | AxisProtocol],
         *,
         weight: str | None = None,
         storage: hist.storage.Storage = hist.storage.Double(),  # noqa: B008
-    ) -> T:
+    ) -> Self:
         axes_list: list[Any] = []
         for ax in axes:
             if isinstance(ax, str):
@@ -201,28 +207,32 @@ class BaseHist(bh.Histogram, metaclass=MetaConstructor, family=hist):
         self.fill(**data_list, weight=weight_arr)  # type: ignore[arg-type]
         return self
 
-    def project(self: T, *args: int | str) -> T | float | bh.accumulators.Accumulator:
+    def project(self, *args: int | str) -> Self | float | bh.accumulators.Accumulator:
         """
         Projection of axis idx.
         """
         int_args = [self._name_to_index(a) if isinstance(a, str) else a for a in args]
         return super().project(*int_args)
 
+    @property
+    def T(self) -> Self:
+        return self.project(*reversed(range(self.ndim)))  # type: ignore[return-value]
+
     def fill(
-        self: T,
+        self,
         *args: ArrayLike,
         weight: ArrayLike | None = None,
         sample: ArrayLike | None = None,
         threads: int | None = None,
         **kwargs: ArrayLike,
-    ) -> T:
+    ) -> Self:
         """
         Insert data into the histogram using names and indices, return
         a Hist object.
         """
 
         data_dict = {
-            self._name_to_index(k) if isinstance(k, str) else k: v
+            self._name_to_index(k) if isinstance(k, str) else k: v  # type: ignore[redundant-expr]
             for k, v in kwargs.items()
         }
 
@@ -233,17 +243,82 @@ class BaseHist(bh.Histogram, metaclass=MetaConstructor, family=hist):
 
         data = (data_dict[i] for i in range(len(args), self.ndim))
 
-        total_data = tuple(args) + tuple(data)  # Python 2 can't unpack twice
-        return super().fill(*total_data, weight=weight, sample=sample, threads=threads)
+        return super().fill(*args, *data, weight=weight, sample=sample, threads=threads)
+
+    def fill_flattened(
+        self: Self,
+        *args: Any,
+        weight: Any | None = None,
+        sample: Any | None = None,
+        threads: int | None = None,
+        **kwargs: Any,
+    ) -> Self:
+        axis_names = {ax.name for ax in self.axes}
+
+        non_user_kwargs = {}
+        if weight is not None:
+            non_user_kwargs["weight"] = weight
+        if sample is not None:
+            non_user_kwargs["sample"] = sample
+
+        # Single arguments are either arrays for single-dimensional histograms, or
+        # structures for multi-dimensional hists that must first be unpacked
+        if len(args) == 1 and not kwargs:
+            (arg,) = args
+            destructured = interop.destructure(arg)
+            # Try to unpack the array, if it's valid to do so, i.e. is the Awkward Array a record array?
+            if destructured is None:
+                # Can't unpack, fall back on broadcasting single array (to flatten and convert)
+                broadcast = interop.broadcast_and_flatten(
+                    (arg, *non_user_kwargs.values())
+                )
+                # Partition out non-user args
+                user_args_broadcast = broadcast[:1]
+                user_kwargs_broadcast = {}
+                non_user_kwargs_broadcast = dict(
+                    zip(non_user_kwargs.keys(), broadcast[1:])
+                )
+            else:
+                # Result must be broadcast, so unpack and rebuild
+                broadcast = interop.broadcast_and_flatten(
+                    (*destructured.values(), *non_user_kwargs.values())
+                )
+                # Partition into user and non-user args
+                user_args_broadcast = ()
+                user_kwargs_broadcast = {
+                    k: v
+                    for k, v in zip(destructured, broadcast[: len(destructured)])
+                    if k in axis_names
+                }
+                non_user_kwargs_broadcast = dict(
+                    zip(non_user_kwargs, broadcast[len(destructured) :])
+                )
+        # Multiple args: broadcast and flatten!
+        else:
+            inputs = (*args, *kwargs.values(), *non_user_kwargs)
+            broadcast = interop.broadcast_and_flatten(inputs)
+            user_args_broadcast = broadcast[: len(args)]
+            user_kwargs_broadcast = dict(
+                zip(kwargs, broadcast[len(args) : len(args) + len(kwargs)])
+            )
+            non_user_kwargs_broadcast = dict(
+                zip(non_user_kwargs, broadcast[len(args) + len(kwargs) :])
+            )
+        return self.fill(
+            *user_args_broadcast,
+            **user_kwargs_broadcast,
+            threads=threads,
+            **non_user_kwargs_broadcast,
+        )
 
     def sort(
-        self: T,
+        self,
         axis: int | str,
         key: (
             Callable[[int], SupportsLessThan] | Callable[[str], SupportsLessThan] | None
         ) = None,
         reverse: bool = False,
-    ) -> T:
+    ) -> Self:
         """
         Sort a categorical axis.
         """
@@ -251,7 +326,7 @@ class BaseHist(bh.Histogram, metaclass=MetaConstructor, family=hist):
         with warnings.catch_warnings():
             warnings.simplefilter("ignore")
             sorted_cats = sorted(self.axes[axis], key=key, reverse=reverse)
-            # This can only return T, not float, etc., so we ignore the extra types here
+            # This can only return Self, not float, etc., so we ignore the extra types here
             return self[{axis: [bh.loc(x) for x in sorted_cats]}]  # type: ignore[dict-item, return-value]
 
     def _loc_shortcut(self, x: Any) -> Any:
@@ -316,8 +391,8 @@ class BaseHist(bh.Histogram, metaclass=MetaConstructor, family=hist):
         return tuple(self._loc_shortcut(v) for v in index)  # type: ignore[union-attr, union-attr, union-attr, union-attr]
 
     def __getitem__(  # type: ignore[override]
-        self: T, index: IndexingExpr
-    ) -> T | float | bh.accumulators.Accumulator:
+        self, index: IndexingExpr
+    ) -> Self | float | bh.accumulators.Accumulator:
         """
         Get histogram item.
         """
@@ -333,7 +408,7 @@ class BaseHist(bh.Histogram, metaclass=MetaConstructor, family=hist):
 
         return super().__setitem__(self._index_transform(index), value)
 
-    def profile(self: T, axis: int | str) -> T:
+    def profile(self, axis: int | str) -> Self:
         """
         Returns a profile (Mean/WeightedMean) histogram from a normal histogram
         with N-1 axes. The axis given is profiled over and removed from the
@@ -389,15 +464,14 @@ class BaseHist(bh.Histogram, metaclass=MetaConstructor, family=hist):
         """
         Plot method for BaseHist object.
         """
-        _has_categorical = 0
-        if (
-            sum(t.ordered for t in self.axes.traits) == 1
-            and sum(t.discrete for t in self.axes.traits) == 1
-        ):
-            _has_categorical = 1
+        ordered_traits = sum(t.ordered for t in self.axes.traits)
+        discrete_traits = sum(t.discrete for t in self.axes.traits)
+        _has_categorical = ordered_traits == discrete_traits == 1
         _project = _has_categorical or overlay is not None
+
         if self.ndim == 1 or (self.ndim == 2 and _project):
             return self.plot1d(*args, overlay=overlay, **kwargs)
+
         if self.ndim == 2:
             return self.plot2d(*args, **kwargs)
 
@@ -516,7 +590,6 @@ class BaseHist(bh.Histogram, metaclass=MetaConstructor, family=hist):
         ax: matplotlib.axes.Axes | None = None,
         **kwargs: Any,
     ) -> Any:
-
         from hist import plot
 
         return plot.plot_pie(self, ax=ax, **kwargs)
@@ -535,6 +608,22 @@ class BaseHist(bh.Histogram, metaclass=MetaConstructor, family=hist):
 
         return hist.stack.Stack(*stack_histograms)
 
+    def integrate(
+        self,
+        name: int | str,
+        i_or_list: InnerIndexing | list[str | int] | None = None,
+        j: InnerIndexing | None = None,
+    ) -> Self | float | bh.accumulators.Accumulator:
+        if isinstance(i_or_list, list):
+            with warnings.catch_warnings():
+                warnings.simplefilter("ignore")
+
+                # TODO: We could teach the typing system that list always returns Self type
+                selection: Self = self[{name: i_or_list}]  # type: ignore[assignment, dict-item]
+                return selection[{name: slice(0, len(i_or_list), sum)}]
+
+        return self[{name: slice(i_or_list, j, sum)}]
+
     def sum(self, flow: bool = False) -> float | bh.accumulators.Accumulator:
         """
         Compute the sum over the histogram bins (optionally including the flow bins).
