@@ -8,7 +8,7 @@ import numpy as np
 
 import hist
 
-from .typing import Literal
+from ._compat.typing import Literal
 
 try:
     import matplotlib.axes
@@ -16,7 +16,7 @@ try:
     from matplotlib import patches, transforms
     from mplhep.plot import Hist1DArtists, Hist2DArtists, hist2dplot, histplot
 except ModuleNotFoundError:
-    print(
+    print(  # noqa: T201
         "Hist requires mplhep to plot, either install hist[plot] or mplhep",
         file=sys.stderr,
     )
@@ -32,7 +32,7 @@ __all__ = (
     "plot_stack",
 )
 
-_PLT_MISSING_MSG = "Hist plotting with fitting requires scipy and iminuit. Please install hist[plot] or manually install dependencies."
+_PLT_MISSING_MSG = "Hist plotting with fitting requires scipy and iminuit. Please install hist[plot,fit] or manually install dependencies."
 
 
 class FitResultArtists(NamedTuple):
@@ -114,15 +114,15 @@ def _expr_to_lambda(expr: str) -> Callable[..., Any]:
         tokval = x[1]
         if toknum != NAME:
             continue
-        if ix > 0 and g[ix - 1][1] in ["."]:
+        if ix > 0 and g[ix - 1][1] in {"."}:
             continue
-        if ix < len(g) - 1 and g[ix + 1][1] in [".", "("]:
+        if ix < len(g) - 1 and g[ix + 1][1] in {".", "("}:
             continue
         varnames.append(tokval)
     varnames = list(OrderedDict.fromkeys([name for name in varnames if name != "x"]))
     lambdastr = f"lambda x,{','.join(varnames)}: {expr}"
     # pylint: disable-next=eval-used
-    return eval(lambdastr)  # type: ignore[no-any-return]
+    return eval(lambdastr)  # type: ignore[no-any-return]  # noqa: PGH001
 
 
 def _curve_fit_wrapper(
@@ -140,7 +140,7 @@ def _curve_fit_wrapper(
     try:
         from scipy.optimize import curve_fit, minimize
     except ModuleNotFoundError:
-        print(_PLT_MISSING_MSG, file=sys.stderr)
+        print(_PLT_MISSING_MSG, file=sys.stderr)  # noqa: T201
         raise
 
     params = list(inspect.signature(func).parameters.values())
@@ -162,7 +162,7 @@ def _curve_fit_wrapper(
         try:
             from iminuit import Minuit
         except ModuleNotFoundError:
-            print(_PLT_MISSING_MSG, file=sys.stderr)
+            print(_PLT_MISSING_MSG, file=sys.stderr)  # noqa: T201
             raise
         from scipy.special import gammaln
 
@@ -339,7 +339,7 @@ def _fit_callable_to_hist(
         n_samples = 100
         vopts = np.random.multivariate_normal(popt, pcov, n_samples)
         sampled_ydata = np.vstack([model(xdata, *vopt).T for vopt in vopts])
-        model_uncert = np.nanstd(sampled_ydata, axis=0)  # type: ignore[no-untyped-call]
+        model_uncert = np.nanstd(sampled_ydata, axis=0)
     else:
         model_uncert = np.zeros_like(hist_uncert)
 
@@ -698,7 +698,6 @@ def plot_pie(
     ax: matplotlib.axes.Axes | None = None,
     **kwargs: Any,
 ) -> Any:
-
     if ax is None:
         fig = plt.gcf()
         ax = fig.add_subplot(111)
@@ -717,13 +716,11 @@ def plot_stack(
     legend: bool | None = False,
     **kwargs: Any,
 ) -> Any:
-
     if self[0].ndim != 1:
         raise NotImplementedError("Please project to 1D before calling plot")
 
-    if "label" not in kwargs:
-        if all(h.name is not None for h in self):
-            kwargs["label"] = [h.name for h in self]
+    if "label" not in kwargs and all(h.name is not None for h in self):
+        kwargs["label"] = [h.name for h in self]
 
     ret = histplot(list(self), ax=ax, **kwargs)
     final_ax = ret[0].stairs.axes
